#include "stdio.h"
#include "stdlib.h"
#include <string.h>

#include "io.h"
#include <sys/io.h>

void io_EnableIRQ(void) {
	asm("sti");// only work on Vortex86SX/DX/...
}

void io_DisableIRQ(void){
	asm("cli");// only work on Vortex86SX/DX/...
}
//outport: double word
void io_outpdw(unsigned short addr, unsigned long val) {
    outl(val, addr);
}
//inport: double word
unsigned long io_inpdw(unsigned short addr) {
    return inl(addr);
}
//outport: word
void io_outpw(unsigned short addr, unsigned short val) {
    outw(val, addr);
}
//inport: word
unsigned short io_inpw(unsigned short addr) {
    return inw(addr);
}
//outport: byte
void io_outpb(unsigned short addr, unsigned char val) {
    outb(val, addr);
}
//inport: byte
unsigned char io_inpb(unsigned short addr) {
    return inb(addr);
}

/******************  PCI Configuration Space Access Functions  ****************/
void io_outpcidw(unsigned long addr, unsigned long val) {
    io_DisableIRQ();
	io_outpdw(0x0cf8, addr & 0xfffffffcL);
	io_outpdw(0x0cfc, val);
    io_EnableIRQ();
}

void io_outpciw(unsigned long addr, unsigned short val) {
	int i = (int)(addr & 0x03) * 8;

    io_DisableIRQ();
	io_outpdw(0x0cf8, addr & 0xfffffffcL);
	io_outpdw(0x0cfc, (io_inpdw(0x0cfc) & ~(0x0000ffffL << i)) | ((unsigned long)val << i));
    io_EnableIRQ();
}

void io_outpcib(unsigned long addr, unsigned char val) {
	int i = (int)(addr & 0x03) * 8;

    io_DisableIRQ();
	io_outpdw(0x0cf8, addr & 0xfffffffcL);
	io_outpdw(0x0cfc, (io_inpdw(0x0cfc) & ~(0x000000ffL << i)) | ((unsigned long)val << i));
    io_EnableIRQ();
}

unsigned long io_inpcidw(unsigned long addr) {
    unsigned long tmp;

    io_DisableIRQ();
	io_outpdw(0x0cf8, addr & 0xfffffffcL);
    tmp = io_inpdw(0x0cfc);
    io_EnableIRQ();

    return tmp;
}

unsigned short io_inpciw(unsigned long addr) {
    unsigned short tmp;
	int i = (int)(addr & 0x03) * 8;

    io_DisableIRQ();
	io_outpdw(0x0cf8, addr & 0xfffffffcL);
    tmp = (unsigned short)((io_inpdw(0x0cfc) >> i) & 0xffffL);
    io_EnableIRQ();

    return tmp;
}

unsigned char io_inpcib(unsigned long addr) {
    unsigned char tmp;
	int i = (int)(addr & 0x03) * 8;

    io_DisableIRQ();
	io_outpdw(0x0cf8, addr & 0xfffffffcL);
    tmp = (unsigned char)((io_inpdw(0x0cfc) >> i) & 0xffL);
    io_EnableIRQ();

    return tmp;
}
/*-------------  end of PCI Configuration Space Access Functions  ------------*/

// read south bridge register
unsigned char read_sb_regb(unsigned char idx) {
	return io_inpcib(0x80003800L+(unsigned long)idx);
}

unsigned short read_sb_regw(unsigned char idx) {
	return io_inpciw(0x80003800L+(unsigned long)idx);
}

unsigned long read_sb_reg(unsigned char idx) {
	return io_inpcidw(0x80003800L+(unsigned long)idx);
}

// write south bridge register
void write_sb_regb(unsigned char idx, unsigned char val) {
	io_outpcib(0x80003800L+(unsigned long)idx, val);
}

void write_sb_regw(unsigned char idx, unsigned short val) {
	io_outpciw(0x80003800L+(unsigned long)idx, val);
}

void write_sb_reg(unsigned char idx, unsigned long val) {
	io_outpcidw(0x80003800L+(unsigned long)idx, val);
}

int io_Init(void) {
	if (iopl(3) != 0)
	{
		printf("I/O library fails to initialize");
		return -1;
	}

    return 0;
}
