#include "stdio.h"
#include "stdlib.h"
#include <string.h>

#include "io.h"
#include "pwm.h"

#define SB_PWMSETTING_REG   (0xd0)
#define SB_GPIOFUNC_REG     (0xc8)

#define PWM_LREG_BASEADDR   (BaseAddress + 0x0c)
#define PWM_HREG_BASEADDR   (BaseAddress + 0x10)
#define PWM_CTRL_BASEADDR   (BaseAddress + 0x14)

static unsigned BaseAddress = 0xfe00;

int PWM_init() {
	int i;
	if(io_Init() != 0) {
		printf("IO init fail!!\n");
		return -1;
	}
	
	//switch gpio
	write_sb_reg(SB_GPIOFUNC_REG, 0x00L );
	//set base address with enabling I/O address decoding
	write_sb_reg(SB_PWMSETTING_REG, ((read_sb_reg(SB_PWMSETTING_REG) & 0xff7f0000L) | 0x00800000L) + BaseAddress );
	//clock = 50MHz
	write_sb_reg(SB_PWMSETTING_REG,(read_sb_reg(SB_PWMSETTING_REG) & 0xfeffffffL) + (1L << 24) );
	//disable IRQ
	write_sb_reg(SB_PWMSETTING_REG, read_sb_reg(SB_PWMSETTING_REG) & 0xfff0ffffL );
	//enable pwm & count mode & count = 0
	for(i = 0; i < 16; i++)
		io_outpdw(PWM_CTRL_BASEADDR + i*12, 0x80000000L);
	return 0;
}

int SwitchPIN(int channel, unsigned long mode) {
	if(channel >= 16 || channel < 0) {
		printf("channel %d doesn't exist", channel);
		return -1;
	}
	if(mode == PWM)
		write_sb_reg(SB_GPIOFUNC_REG, read_sb_reg(SB_GPIOFUNC_REG) | (1L << channel) );
	else if(mode == GPIO)
		write_sb_reg(SB_GPIOFUNC_REG, read_sb_reg(SB_GPIOFUNC_REG) & ~(1L << channel));
	else {
		printf("mode is not exist!!\n");
		return -1;
	}
	return 0;
}


int SendPWM(int channel, unsigned long period, unsigned long duty) {
	if ((period == 0L) || (duty == 0L))
	{
		printf("zero period or duty");
		return -1;
	}
	if(channel >= 16 || channel < 0) {
		printf("channel %d doesn't exist", channel);
		return -1;
	}
	
	//Set period & duty
    duty   = duty;//10MHz
    period = period;//10MHz
	io_outpdw(PWM_HREG_BASEADDR + channel*12, duty);
	io_outpdw(PWM_LREG_BASEADDR + channel*12, period - duty);
    //set continue mode
	io_outpdw(PWM_CTRL_BASEADDR + channel*12, 0xc0000000L);
	return 0;
}

void StopPWM(int channel) {
	if(channel >= 16 || channel < 0) {
		printf("channel %d doesn't exist", channel);
		return;
	}
	
	io_outpdw(PWM_CTRL_BASEADDR + channel*12, 0x80000000L + 0x01L );
}

unsigned long IsPWMStop(int channel) {
	return (io_inpdw(PWM_CTRL_BASEADDR + channel*12) & 0x0fffffffL);
}

int PWM_close() {
	int i;
	//switch gpio
	write_sb_reg(SB_GPIOFUNC_REG, 0x00L );
	//disable pwm
	for(i = 0; i < 16; i++)
		io_outpdw(PWM_CTRL_BASEADDR + i*12, 0x00000000L);	
	return 0;
}
